@@ -53,8 +53,9 @@ typedef struct {
 } PnmIOBuffer;
 
 typedef struct {
-	ModuleUpdatedNotifyFunc updated_func;
-	ModulePreparedNotifyFunc prepared_func;
+	GdkPixbufModuleUpdatedFunc updated_func;
+	GdkPixbufModulePreparedFunc prepared_func;
+	GdkPixbufModuleSizeFunc size_func;
 	gpointer user_data;
 	
 	GdkPixbuf *pixbuf;
@@ -81,9 +82,9 @@ typedef struct {
 } PnmLoaderContext;
 
 static GdkPixbuf   *gdk_pixbuf__pnm_image_load          (FILE *f, GError **error);
-static gpointer    gdk_pixbuf__pnm_image_begin_load     (ModuleSizeFunc size_func, 
-                                                         ModulePreparedNotifyFunc func, 
-							 ModuleUpdatedNotifyFunc func2,
+static gpointer    gdk_pixbuf__pnm_image_begin_load     (GdkPixbufModuleSizeFunc size_func, 
+                                                         GdkPixbufModulePreparedFunc func, 
+							 GdkPixbufModuleUpdatedFunc func2,
 							 gpointer user_data,
 							 GError **error);
 static gboolean    gdk_pixbuf__pnm_image_stop_load      (gpointer context, GError **error);
@@ -236,7 +237,7 @@ pnm_read_next_value (PnmIOBuffer *inbuf, guint *value, GError **error)
 	*word = '\0';
 	
 	/* hmmm, there must be more data to this 'word' */
-	if (!g_ascii_isspace (*p) && (*p != '#')  && (p - inptr < 128))
+	if (p == inend || (!g_ascii_isspace (*p) && (*p != '#')  && (p - inptr < 128)))
 	    return PNM_SUSPEND;
 	
 	/* get the value */
@@ -392,7 +393,7 @@ pnm_read_header (PnmLoaderContext *context)
 				g_set_error (context->error,
 					     GDK_PIXBUF_ERROR,
 					     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
- 					     _("Can't handle PNM files with maximum color values greater than 255"));
+ 					     _("Cannot handle PNM files with maximum color values greater than 255"));
 				return PNM_FATAL_ERR;
 			}
 		}
@@ -766,7 +767,7 @@ gdk_pixbuf__pnm_image_load (FILE *f, GError **error)
 				g_set_error (error,
 					     GDK_PIXBUF_ERROR,
 					     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-					     _("Can't allocate memory for loading PNM image"));
+					     _("Cannot allocate memory for loading PNM image"));
 				return NULL;
 			}
 
@@ -804,9 +805,9 @@ gdk_pixbuf__pnm_image_load (FILE *f, GError **error)
  */
 
 static gpointer
-gdk_pixbuf__pnm_image_begin_load (ModuleSizeFunc size_func, 
-                                  ModulePreparedNotifyFunc prepared_func, 
-				  ModuleUpdatedNotifyFunc  updated_func,
+gdk_pixbuf__pnm_image_begin_load (GdkPixbufModuleSizeFunc size_func, 
+                                  GdkPixbufModulePreparedFunc prepared_func, 
+				  GdkPixbufModuleUpdatedFunc  updated_func,
 				  gpointer user_data,
 				  GError **error)
 {
@@ -820,6 +821,7 @@ gdk_pixbuf__pnm_image_begin_load (ModuleSizeFunc size_func,
 		return NULL;
 	}
 	memset (context, 0, sizeof (PnmLoaderContext));
+	context->size_func = size_func;
 	context->prepared_func = prepared_func;
 	context->updated_func  = updated_func;
 	context->user_data = user_data;
@@ -944,6 +946,16 @@ gdk_pixbuf__pnm_image_load_increment (gpointer data,
 			
 			context->got_header = TRUE;
 		}
+
+		if (context->size_func) {
+			gint w = context->width;
+			gint h = context->height;
+			(*context->size_func) (&w, &h, context->user_data);
+			
+			if (w == 0 || h == 0) 
+				return FALSE;
+		}
+		
 		
 		/* scan until we hit image data */
 		if (!context->did_prescan) {
@@ -1030,10 +1042,46 @@ gdk_pixbuf__pnm_image_load_increment (gpointer data,
 }
 
 void
-gdk_pixbuf__pnm_fill_vtable (GdkPixbufModule *module)
+MODULE_ENTRY (pnm, fill_vtable) (GdkPixbufModule *module)
 {
-  module->load = gdk_pixbuf__pnm_image_load;
-  module->begin_load = gdk_pixbuf__pnm_image_begin_load;
-  module->stop_load = gdk_pixbuf__pnm_image_stop_load;
-  module->load_increment = gdk_pixbuf__pnm_image_load_increment;
+	module->load = gdk_pixbuf__pnm_image_load;
+	module->begin_load = gdk_pixbuf__pnm_image_begin_load;
+	module->stop_load = gdk_pixbuf__pnm_image_stop_load;
+	module->load_increment = gdk_pixbuf__pnm_image_load_increment;
+}
+
+void
+MODULE_ENTRY (pnm, fill_info) (GdkPixbufFormat *info)
+{
+	static GdkPixbufModulePattern signature[] = {
+		{ "P1", NULL, 100 },
+		{ "P2", NULL, 100 },
+		{ "P3", NULL, 100 },
+		{ "P4", NULL, 100 },
+		{ "P5", NULL, 100 },
+		{ "P6", NULL, 100 },
+		{ NULL, NULL, 0 }
+	};
+	static gchar * mime_types[] = {
+		"image/x-portable-anymap",
+		"image/x-portable-bitmap",
+		"image/x-portable-graymap",
+		"image/x-portable-pixmap",
+		NULL
+	};
+	static gchar * extensions[] = {
+		"pnm",
+		"pbm",
+		"pgm",
+		"ppm",
+		NULL
+	};
+
+	info->name = "pnm";
+	info->signature = signature;
+	info->description = N_("The PNM/PBM/PGM/PPM image format family");
+	info->mime_types = mime_types;
+	info->extensions = extensions;
+	info->flags = 0;
+	info->license = "LGPL";
 }
