@@ -36,6 +36,7 @@
  * - Error handling was designed to be very paranoid.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -137,6 +138,7 @@ struct _TGAContext {
 	gboolean prepared;
 	gboolean done;
 
+	GdkPixbufModuleSizeFunc sfunc;
 	GdkPixbufModulePreparedFunc pfunc;
 	GdkPixbufModuleUpdatedFunc ufunc;
 	gpointer udata;
@@ -154,7 +156,7 @@ static IOBuffer *io_buffer_new(GError **err)
 	if (!buffer) {
 		g_set_error(err, GDK_PIXBUF_ERROR,
 			    GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-			    _("Can't allocate memory for IOBuffer struct"));
+			    _("Cannot allocate memory for IOBuffer struct"));
 		return NULL;
 	}
 	buffer->data = NULL;
@@ -173,7 +175,7 @@ static IOBuffer *io_buffer_append(IOBuffer *buffer,
 		if (!buffer->data) {
 			g_set_error(err, GDK_PIXBUF_ERROR,
 				    GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-				    _("Can't allocate memory for IOBuffer data"));
+				    _("Cannot allocate memory for IOBuffer data"));
 			g_free(buffer);
 			return NULL;
 		}
@@ -184,7 +186,7 @@ static IOBuffer *io_buffer_append(IOBuffer *buffer,
 		if (!tmp) {
 			g_set_error(err, GDK_PIXBUF_ERROR,
 				    GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-				    _("Can't realloc IOBuffer data"));
+				    _("Cannot realloc IOBuffer data"));
 			g_free(buffer);
 			return NULL;
 		}
@@ -214,7 +216,7 @@ static IOBuffer *io_buffer_free_segment(IOBuffer *buffer,
 		if (!new_buf) {
 			g_set_error(err, GDK_PIXBUF_ERROR,
 				    GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-				    _("Can't allocate temporary IOBuffer data"));
+				    _("Cannot allocate temporary IOBuffer data"));
 			g_free(buffer->data);
 			g_free(buffer);
 			return NULL;
@@ -334,11 +336,21 @@ static gboolean fill_in_context(TGAContext *ctx, GError **err)
 	w = LE16(ctx->hdr->width);
 	h = LE16(ctx->hdr->height);
 
+	if (ctx->sfunc) {
+		gint wi = w;
+		gint hi = h;
+		
+		(*ctx->sfunc) (&wi, &hi, ctx->udata);
+		
+		if (wi == 0 || hi == 0) 
+			return FALSE;
+	}
+
 	ctx->pbuf = get_contiguous_pixbuf (w, h, alpha);
 
 	if (!ctx->pbuf) {
 		g_set_error(err, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-			    _("Can't allocate new pixbuf"));
+			    _("Cannot allocate new pixbuf"));
 		return FALSE;
 	}
 
@@ -677,14 +689,14 @@ static gboolean try_colormap(TGAContext *ctx, GError **err)
 	ctx->cmap = g_try_malloc(sizeof(TGAColormap));
 	if (!ctx->cmap) {
 		g_set_error(err, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-			    _("Can't allocate colormap structure"));
+			    _("Cannot allocate colormap structure"));
 		return FALSE;
 	}
 	ctx->cmap->size = LE16(ctx->hdr->cmap_n_colors);
 	ctx->cmap->cols = g_try_malloc(sizeof(TGAColor) * ctx->cmap->size);
 	if (!ctx->cmap->cols) {
 		g_set_error(err, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-			    _("Can't allocate colormap entries"));
+			    _("Cannot allocate colormap entries"));
 		return FALSE;
 	}
 
@@ -724,7 +736,7 @@ static gboolean try_preload(TGAContext *ctx, GError **err)
 			if (!ctx->hdr) {
 				g_set_error(err, GDK_PIXBUF_ERROR,
 					    GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-					    _("Can't allocate TGA header memory"));
+					    _("Cannot allocate TGA header memory"));
 				return FALSE;
 			}
 			g_memmove(ctx->hdr, ctx->in->data, sizeof(TGAHeader));
@@ -846,7 +858,7 @@ static gpointer gdk_pixbuf__tga_begin_load(GdkPixbufModuleSizeFunc f0,
 	if (!ctx) {
 		g_set_error(err, GDK_PIXBUF_ERROR, 
 			    GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-			    _("Can't allocate memory for TGA context struct"));
+			    _("Cannot allocate memory for TGA context struct"));
 		return NULL;
 	}
 
@@ -872,6 +884,7 @@ static gpointer gdk_pixbuf__tga_begin_load(GdkPixbufModuleSizeFunc f0,
 	ctx->prepared = FALSE;
 	ctx->done = FALSE;
 
+	ctx->sfunc = f0;
 	ctx->pfunc = f1;
 	ctx->ufunc = f2;
 	ctx->udata = udata;
@@ -983,4 +996,5 @@ MODULE_ENTRY (tga, fill_info) (GdkPixbufFormat *info)
 	info->mime_types = mime_types;
 	info->extensions = extensions;
 	info->flags = 0;
+	info->license = "LGPL";
 }
