@@ -197,6 +197,7 @@ gdk_event_type_get_type (void)
       { GDK_SCROLL, "GDK_SCROLL", "scroll" },
       { GDK_WINDOW_STATE, "GDK_WINDOW_STATE", "window-state" },
       { GDK_SETTING, "GDK_SETTING", "setting" },
+      { GDK_OWNER_CHANGE, "GDK_OWNER_CHANGE", "owner-change" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GdkEventType", values);
@@ -358,6 +359,22 @@ gdk_setting_action_get_type (void)
   return etype;
 }
 
+GType
+gdk_owner_change_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GDK_OWNER_CHANGE_NEW_OWNER, "GDK_OWNER_CHANGE_NEW_OWNER", "new-owner" },
+      { GDK_OWNER_CHANGE_DESTROY, "GDK_OWNER_CHANGE_DESTROY", "destroy" },
+      { GDK_OWNER_CHANGE_CLOSE, "GDK_OWNER_CHANGE_CLOSE", "close" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GdkOwnerChange", values);
+  }
+  return etype;
+}
+
 
 /* enumerations from "gdkfont.h" */
 GType
