@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "gtkcomboboxentry.h"
 #include "gtkcelllayout.h"
 
@@ -53,6 +54,9 @@ static void gtk_combo_box_entry_contents_changed (GtkEntry              *entry,
                                                   gpointer               user_data);
 static gboolean gtk_combo_box_entry_mnemonic_activate (GtkWidget        *entry,
 						       gboolean          group_cycling);
+static void has_frame_changed                    (GtkComboBoxEntry      *entry_box,
+						  GParamSpec            *pspec,
+						  gpointer               data);
 
 enum
 {
@@ -124,6 +128,9 @@ gtk_combo_box_entry_init (GtkComboBoxEntry *entry_box)
   entry_box->priv->text_column = -1;
 
   entry_box->priv->entry = gtk_entry_new ();
+  /* this flag is a hack to tell the entry to fill its allocation.
+   */
+  GTK_ENTRY (entry_box->priv->entry)->is_cell_renderer = TRUE;
   gtk_container_add (GTK_CONTAINER (entry_box), entry_box->priv->entry);
   gtk_widget_show (entry_box->priv->entry);
 
@@ -138,6 +145,8 @@ gtk_combo_box_entry_init (GtkComboBoxEntry *entry_box)
                     entry_box);
   g_signal_connect (entry_box, "changed",
                     G_CALLBACK (gtk_combo_box_entry_active_changed), NULL);
+  has_frame_changed (entry_box, NULL, NULL);
+  g_signal_connect (entry_box, "notify::has-frame", G_CALLBACK (has_frame_changed), NULL);
 }
 
 static void
@@ -184,36 +193,41 @@ static void
 gtk_combo_box_entry_active_changed (GtkComboBox *combo_box,
                                     gpointer     user_data)
 {
-  gint index;
   GtkComboBoxEntry *entry_box = GTK_COMBO_BOX_ENTRY (combo_box);
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gchar *str = NULL;
 
-  index = gtk_combo_box_get_active (combo_box);
-
-  g_signal_handlers_block_by_func (entry_box->priv->entry,
-                                   gtk_combo_box_entry_contents_changed,
-                                   combo_box);
-
-  if (index >= 0)
+  if (gtk_combo_box_get_active_iter (combo_box, &iter))
     {
-      gchar *str = NULL;
-      GtkTreeIter iter;
-      GtkTreeModel *model;
+      g_signal_handlers_block_by_func (entry_box->priv->entry,
+				       gtk_combo_box_entry_contents_changed,
+				       combo_box);
 
       model = gtk_combo_box_get_model (combo_box);
 
-      gtk_tree_model_iter_nth_child (model, &iter, NULL, index);
-      gtk_tree_model_get (model, &iter,
-                          entry_box->priv->text_column, &str,
-                          -1);
-
+      gtk_tree_model_get (model, &iter, 
+			  entry_box->priv->text_column, &str, 
+			  -1);
       gtk_entry_set_text (GTK_ENTRY (entry_box->priv->entry), str);
-
       g_free (str);
-    }
 
-  g_signal_handlers_unblock_by_func (entry_box->priv->entry,
-                                     gtk_combo_box_entry_contents_changed,
-                                     combo_box);
+      g_signal_handlers_unblock_by_func (entry_box->priv->entry,
+					 gtk_combo_box_entry_contents_changed,
+					 combo_box);
+    }
+}
+
+static void 
+has_frame_changed (GtkComboBoxEntry *entry_box,
+		   GParamSpec       *pspec,
+		   gpointer          data)
+{
+  gboolean has_frame;
+  
+  g_object_get (entry_box, "has_frame", &has_frame, NULL);
+
+  gtk_entry_set_has_frame (GTK_ENTRY (entry_box->priv->entry), has_frame);
 }
 
 static void
@@ -362,8 +376,8 @@ gtk_combo_box_entry_new_text (void)
   GtkListStore *store;
 
   store = gtk_list_store_new (1, G_TYPE_STRING);
-
   entry_box = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL (store), 0);
+  g_object_unref (store);
 
   return entry_box;
 }
