@@ -34,6 +34,8 @@
 #include <glib/gprintf.h>
 #include <string.h>
 
+#include <atk/atk.h>
+
 #include "gdk/gdk.h"
 #include "gdk/gdkkeysyms.h"
 
@@ -55,6 +57,7 @@
 #include "gtkvbox.h"
 #include "gtkscrolledwindow.h"
 #include "gtkintl.h"
+#include "gtkaccessible.h"
 
 /* We don't enable the font and style entries because they don't add
  * much in terms of visible effect and have a weird effect on keynav.
@@ -279,6 +282,28 @@ static void gtk_font_selection_get_property (GObject         *object,
     }
 }
 
+/* Handles key press events on the lists, so that we can trap Enter to
+ * activate the default button on our own.
+ */
+static gboolean
+list_row_activated (GtkWidget *widget)
+{
+  GtkWindow *window;
+  
+  window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (widget)));
+  if (!GTK_WIDGET_TOPLEVEL (window))
+    window = NULL;
+  
+  if (window
+      && widget != window->default_widget
+      && !(widget == window->focus_widget &&
+	   (!window->default_widget || !GTK_WIDGET_SENSITIVE (window->default_widget))))
+    {
+      gtk_window_activate_default (window);
+    }
+  
+  return TRUE;
+}
 
 static void
 gtk_font_selection_init (GtkFontSelection *fontsel)
@@ -291,6 +316,7 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
   GtkListStore *model;
   GtkTreeViewColumn *column;
   GList *focus_chain = NULL;
+  AtkObject *atk_obj;
 
   gtk_widget_push_composite_child ();
 
@@ -359,6 +385,9 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
   fontsel->family_list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
   g_object_unref (model);
 
+  g_signal_connect (fontsel->family_list, "row-activated",
+		    G_CALLBACK (list_row_activated), fontsel);
+
   column = gtk_tree_view_column_new_with_attributes ("Family",
 						     gtk_cell_renderer_text_new (),
 						     "text", FAMILY_NAME_COLUMN,
@@ -392,6 +421,8 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
 			      G_TYPE_STRING); /* FACE_NAME_COLUMN */
   fontsel->face_list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
   g_object_unref (model);
+  g_signal_connect (fontsel->face_list, "row-activated",
+		    G_CALLBACK (list_row_activated), fontsel);
 
   gtk_label_set_mnemonic_widget (GTK_LABEL (style_label), fontsel->face_list);
 
@@ -425,6 +456,8 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
   model = gtk_list_store_new (1, G_TYPE_INT);
   fontsel->size_list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
   g_object_unref (model);
+  g_signal_connect (fontsel->size_list, "row-activated",
+		    G_CALLBACK (list_row_activated), fontsel);
 
   column = gtk_tree_view_column_new_with_attributes ("Size",
 						     gtk_cell_renderer_text_new (),
@@ -465,6 +498,50 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
 
   g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (fontsel->size_list)), "changed",
 		    G_CALLBACK (gtk_font_selection_select_size), fontsel);
+  atk_obj = gtk_widget_get_accessible (fontsel->size_list);
+  if (GTK_IS_ACCESSIBLE (atk_obj))
+    {
+      /* Accessibility support is enabled.
+       * Make the label ATK_RELATON_LABEL_FOR for the size list as well.
+       */
+      AtkObject *atk_label;
+      AtkRelationSet *relation_set;
+      AtkRelation *relation;
+      AtkObject *obj_array[1];
+      GPtrArray *array;
+
+      atk_label = gtk_widget_get_accessible (label);
+      relation_set = atk_object_ref_relation_set (atk_obj);
+      relation = atk_relation_set_get_relation_by_type (relation_set, ATK_RELATION_LABELLED_BY);
+      if (relation)
+        {
+          array = atk_relation_get_target (relation);
+          g_ptr_array_add (array, atk_label);
+        }
+      else 
+        {
+          obj_array[0] = atk_label;
+          relation = atk_relation_new (obj_array, 1, ATK_RELATION_LABELLED_BY);
+          atk_relation_set_add (relation_set, relation);
+        }
+      g_object_unref (relation_set);
+
+      relation_set = atk_object_ref_relation_set (atk_label);
+      relation = atk_relation_set_get_relation_by_type (relation_set, ATK_RELATION_LABEL_FOR);
+      if (relation)
+        {
+          array = atk_relation_get_target (relation);
+          g_ptr_array_add (array, atk_obj);
+        }
+      else 
+        {
+          obj_array[0] = atk_obj;
+          relation = atk_relation_new (obj_array, 1, ATK_RELATION_LABEL_FOR);
+          atk_relation_set_add (relation_set, relation);
+        }
+      g_object_unref (relation_set);
+    }    
+      
 
   /* create the text entry widget */
   label = gtk_label_new_with_mnemonic (_("_Preview:"));
@@ -520,8 +597,7 @@ gtk_font_selection_finalize (GObject *object)
   if (fontsel->font)
     gdk_font_unref (fontsel->font);
   
-  if (G_OBJECT_CLASS (font_selection_parent_class)->finalize)
-    (* G_OBJECT_CLASS (font_selection_parent_class)->finalize) (object);
+  (* G_OBJECT_CLASS (font_selection_parent_class)->finalize) (object);
 }
 
 static void
@@ -958,7 +1034,10 @@ gtk_font_selection_size_activate (GtkWidget   *w,
   text = gtk_entry_get_text (GTK_ENTRY (fontsel->size_entry));
   new_size = MAX (0.1, atof (text) * PANGO_SCALE + 0.5);
 
-  gtk_font_selection_set_size (fontsel, new_size);
+  if (fontsel->size != new_size)
+    gtk_font_selection_set_size (fontsel, new_size);
+  else 
+    list_row_activated (w);
 }
 
 static gboolean
@@ -966,7 +1045,16 @@ gtk_font_selection_size_focus_out (GtkWidget     *w,
 				   GdkEventFocus *event,
 				   gpointer       data)
 {
-  gtk_font_selection_size_activate (w, data);
+  GtkFontSelection *fontsel;
+  gint new_size;
+  const gchar *text;
+  
+  fontsel = GTK_FONT_SELECTION (data);
+
+  text = gtk_entry_get_text (GTK_ENTRY (fontsel->size_entry));
+  new_size = MAX (0.1, atof (text) * PANGO_SCALE + 0.5);
+
+  gtk_font_selection_set_size (fontsel, new_size);
   
   return TRUE;
 }
