@@ -24,6 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
+
 #include "gdkconfig.h"
 
 #include <locale.h>
@@ -48,20 +50,20 @@
 
 #include <pango/pango-utils.h>	/* For pango_split_file_list */
 
+#include "gtkintl.h"
+
 #include "gtkaccelmap.h"
+#include "gtkbox.h"
 #include "gtkdnd.h"
 #include "gtkversion.h"
 #include "gtkmain.h"
 #include "gtkrc.h"
 #include "gtkselection.h"
 #include "gtksettings.h"
-#include "gtksignal.h"
 #include "gtkwidget.h"
 #include "gtkwindow.h"
 #include "gtkprivate.h"
-#include "config.h"
 #include "gtkdebug.h"
-#include "gtkintl.h"
 
 /* Private type definitions
  */
@@ -69,6 +71,7 @@ typedef struct _GtkInitFunction		 GtkInitFunction;
 typedef struct _GtkQuitFunction		 GtkQuitFunction;
 typedef struct _GtkClosure	         GtkClosure;
 typedef struct _GtkKeySnooperData	 GtkKeySnooperData;
+typedef struct _GtkModuleInfo            GtkModuleInfo;
 
 struct _GtkInitFunction
 {
@@ -100,6 +103,12 @@ struct _GtkKeySnooperData
   guint id;
 };
 
+struct _GtkModuleInfo
+{
+  GtkModuleInitFunc init_func;
+  GtkModuleDisplayInitFunc display_init_func;
+};
+
 static gint  gtk_quit_invoke_function	 (GtkQuitFunction    *quitf);
 static void  gtk_quit_destroy		 (GtkQuitFunction    *quitf);
 static gint  gtk_invoke_key_snoopers	 (GtkWidget	     *grab_widget,
@@ -126,6 +135,13 @@ const guint gtk_micro_version = GTK_MICRO_VERSION;
 const guint gtk_binary_age = GTK_BINARY_AGE;
 const guint gtk_interface_age = GTK_INTERFACE_AGE;
 
+static GSList *gtk_modules;
+
+/* Saved argc,argv for delayed module initialization
+ */
+static gint    gtk_argc = 0;
+static gchar **gtk_argv = NULL;
+
 static guint gtk_main_loop_level = 0;
 static gint gtk_initialized = FALSE;
 static GList *current_events = NULL;
@@ -140,13 +156,6 @@ static GMemChunk *quit_mem_chunk = NULL;
 
 static GSList *key_snoopers = NULL;
 
-static GdkVisual *gtk_visual;		   /* The visual to be used in creating new
-					    *  widgets.
-					    */
-static GdkColormap *gtk_colormap;	   /* The colormap to be used in creating new
-					    *  widgets.
-					    */
-
 guint gtk_debug_flags = 0;		   /* Global GTK debug flag */
 
 #ifdef G_ENABLE_DEBUG
@@ -156,35 +165,67 @@ static const GDebugKey gtk_debug_keys[] = {
   {"text", GTK_DEBUG_TEXT},
   {"tree", GTK_DEBUG_TREE},
   {"updates", GTK_DEBUG_UPDATES},
-  {"keybindings", GTK_DEBUG_KEYBINDINGS}
+  {"keybindings", GTK_DEBUG_KEYBINDINGS},
+  {"multihead", GTK_DEBUG_MULTIHEAD}
 };
 
 static const guint gtk_ndebug_keys = sizeof (gtk_debug_keys) / sizeof (GDebugKey);
 
 #endif /* G_ENABLE_DEBUG */
 
+/**
+ * gtk_check_version:
+ * @required_major: the required major version.
+ * @required_minor: the required major version.
+ * @required_micro: the required major version.
+ * 
+ * Checks that the GTK+ library in use is compatible with the
+ * given version. Generally you would pass in the constants
+ * #GTK_MAJOR_VERSION, #GTK_MINOR_VERSION, #GTK_MICRO_VERSION
+ * as the three arguments to this function; that produces
+ * a check that the library in use is compatible with
+ * the version of GTK+ the application or module was compiled
+ * against.
+ *
+ * Compatibility is defined by two things: first the version
+ * of the running library is newer than the version
+ * @required_major.required_minor.@required_micro. Second
+ * the running library must be binary compatible with the
+ * version @required_major.required_minor.@required_micro
+ * (same major version.)
+ *
+ * This function is primarily for GTK+ modules; the module
+ * can call this function to check that it wasn't loaded
+ * into an incompatible version of GTK+. However, such a
+ * a check isn't completely reliable, since the module may be
+ * linked against an old version of GTK+ and calling the
+ * old version of gtk_check_version(), but still get loaded
+ * into an application using a newer version of GTK+.
+ *
+ * Return value: %NULL if the GTK+ library is compatible with the
+ *   given version, or a string describing the version mismatch.
+ *   The returned string is owned by GTK+ and should not be modified
+ *   or freed.
+ **/
 gchar*
 gtk_check_version (guint required_major,
 		   guint required_minor,
 		   guint required_micro)
 {
+  gint gtk_effective_micro = 100 * GTK_MINOR_VERSION + GTK_MICRO_VERSION;
+  gint required_effective_micro = 100 * required_minor + required_micro;
+
   if (required_major > GTK_MAJOR_VERSION)
     return "Gtk+ version too old (major mismatch)";
   if (required_major < GTK_MAJOR_VERSION)
     return "Gtk+ version too new (major mismatch)";
-  if (required_minor > GTK_MINOR_VERSION)
-    return "Gtk+ version too old (minor mismatch)";
-  if (required_minor < GTK_MINOR_VERSION)
-    return "Gtk+ version too new (minor mismatch)";
-  if (required_micro < GTK_MICRO_VERSION - GTK_BINARY_AGE)
+  if (required_effective_micro < gtk_effective_micro - GTK_BINARY_AGE)
     return "Gtk+ version too new (micro mismatch)";
-  if (required_micro > GTK_MICRO_VERSION)
+  if (required_effective_micro > gtk_effective_micro)
     return "Gtk+ version too old (micro mismatch)";
   return NULL;
 }
 
-#undef gtk_init_check
-
 /* This checks to see if the process is running suid or sgid
  * at the current time. If so, we don't allow GTK+ to be initialized.
  * This is meant to be a mild check - we only error out if we
@@ -234,6 +275,17 @@ check_setugid (void)
 
 G_WIN32_DLLMAIN_FOR_DLL_NAME(static, dll_name)
 
+const gchar *
+_gtk_get_datadir (void)
+{
+  static char *gtk_datadir = NULL;
+  if (gtk_datadir == NULL)
+    gtk_datadir = g_win32_get_package_installation_subdirectory
+      (GETTEXT_PACKAGE, dll_name, "share");
+
+  return gtk_datadir;
+}
+
 const gchar *
 _gtk_get_libdir (void)
 {
@@ -407,8 +459,8 @@ module_build_la_path (const gchar *directory,
  * @name: the name of the module
  * @type: the type of the module, for instance 'modules', 'engines', immodules'
  * 
- * Looks for a dynamically module named @name of type @type in the standard GTK+
- *  module search path.
+ * Looks for a dynamically loadable module named @name of type @type in the
+ * standard GTK+ module search path.
  * 
  * Return value: the pathname to the found module, or %NULL if it wasn't found.
  *  Free with g_free().
@@ -446,15 +498,13 @@ _gtk_find_module (const gchar *name,
       g_free(tmp_name);
     }
 
-  g_strfreev (paths);
-
  found:
+  g_strfreev (paths);
   return module_name;
 }
 
 static GModule *
-find_module (gchar      **module_path,
-	     const gchar *name)
+find_module (const gchar *name)
 {
   GModule *module;
   gchar *module_name;
@@ -475,24 +525,30 @@ find_module (gchar      **module_path,
 }
 
 static GSList *
-load_module (GSList      *gtk_modules,
-	     gchar      **module_path,
+load_module (GSList      *module_list,
 	     const gchar *name)
 {
   GtkModuleInitFunc modinit_func = NULL;
+  GtkModuleInfo *info;
   GModule *module = NULL;
   
   if (g_module_supported ())
     {
-      module = find_module (module_path, name);
+      module = find_module (name);
       if (module &&
 	  g_module_symbol (module, "gtk_module_init", (gpointer *) &modinit_func) &&
 	  modinit_func)
 	{
-	  if (!g_slist_find (gtk_modules, modinit_func))
+	  if (!g_slist_find (module_list, (gconstpointer) modinit_func))
 	    {
 	      g_module_make_resident (module);
-	      gtk_modules = g_slist_prepend (gtk_modules, modinit_func);
+	      info = g_new (GtkModuleInfo, 1);
+
+	      info->init_func = modinit_func;
+	      g_module_symbol (module, "gtk_module_display_init",
+			       (gpointer *) &info->display_init_func);
+	      
+	      module_list = g_slist_prepend (module_list, info);
 	    }
 	  else
 	    {
@@ -510,26 +566,24 @@ load_module (GSList      *gtk_modules,
 	g_module_close (module);
     }
   
-  return gtk_modules;
+  return module_list;
 }
 
 static GSList *
 load_modules (const char *module_str)
 {
-  gchar **module_path = get_module_path ();
   gchar **module_names = pango_split_file_list (module_str);
-  GSList *gtk_modules = NULL;
+  GSList *module_list = NULL;
   gint i;
   
   for (i = 0; module_names[i]; i++)
-    gtk_modules = load_module (gtk_modules, module_path, module_names[i]);
+    module_list = load_module (module_list, module_names[i]);
   
-  gtk_modules = g_slist_reverse (gtk_modules);
+  module_list = g_slist_reverse (module_list);
   
   g_strfreev (module_names);
-  g_strfreev (module_path);
 
-  return gtk_modules;
+  return module_list;
 }
 
 static gboolean do_setlocale = TRUE;
@@ -554,13 +608,77 @@ gtk_disable_setlocale (void)
   do_setlocale = FALSE;
 }
 
+#undef gtk_init_check
+
+static void
+default_display_notify_cb (GdkDisplayManager *display_manager)
+{
+  GSList *slist;
+
+  /* Initialize non-multihead-aware modules when the
+   * default display is first set to a non-NULL value.
+   */
+  static gboolean initialized = FALSE;
+
+  if (!gdk_display_get_default () || initialized)
+    return;
+
+  initialized = TRUE;
+
+  for (slist = gtk_modules; slist; slist = slist->next)
+    {
+      if (slist->data)
+	{
+	  GtkModuleInfo *info = slist->data;
+
+	  if (!info->display_init_func)
+	    info->init_func (&gtk_argc, &gtk_argv);
+	}
+    }
+}
+
+static void
+display_opened_cb (GdkDisplayManager *display_manager,
+		   GdkDisplay        *display)
+{
+  GSList *slist;
+  
+  for (slist = gtk_modules; slist; slist = slist->next)
+    {
+      if (slist->data)
+	{
+	  GtkModuleInfo *info = slist->data;
+
+	  if (info->display_init_func)
+	    info->display_init_func (display);
+	}
+    }
+}
+
+/**
+ * gtk_parse_args:
+ * @argc: a pointer to the number of command line arguments.
+ * @argv: a pointer to the array of command line arguments.
+ * 
+ * Parses command line arguments, and initializes global
+ * attributes of GTK+, but does not actually open a connection
+ * to a display. (See gdk_display_open(), gdk_get_display_arg_name())
+ *
+ * Any arguments used by GTK+ or GDK are removed from the array and
+ * @argc and @argv are updated accordingly.
+ *
+ * You shouldn't call this function explicitely if you are using
+ * gtk_init(), or gtk_init_check().
+ *
+ * Return value: %TRUE if initialization succeeded, otherwise %FALSE.
+ **/
 gboolean
-gtk_init_check (int	 *argc,
-		char   ***argv)
+gtk_parse_args (int    *argc,
+		char ***argv)
 {
   GString *gtk_modules_string = NULL;
-  GSList *gtk_modules = NULL;
   GSList *slist;
+  GdkDisplayManager *display_manager;
   const gchar *env_string;
 
   if (gtk_initialized)
@@ -581,13 +699,8 @@ gtk_init_check (int	 *argc,
       if (!setlocale (LC_ALL, ""))
 	g_warning ("Locale not supported by C library.\n\tUsing the fallback 'C' locale.");
     }
-  
-  /* Initialize "gdk". We pass along the 'argc' and 'argv'
-   *  parameters as they contain information that GDK uses
-   */
-  if (!gdk_init_check (argc, argv))
-    return FALSE;
 
+  gdk_parse_args (argc, argv);
   gdk_event_handler_set ((GdkEventFunc)gtk_main_do_event, NULL, NULL);
   
 #ifdef G_ENABLE_DEBUG
@@ -706,6 +819,11 @@ gtk_init_check (int	 *argc,
 	      *argc -= k;
 	    }
 	}
+
+      gtk_argv = g_malloc ((gtk_argc + 1) * sizeof (char*));
+      for (i = 0; i < gtk_argc; i++)
+	gtk_argv[i] = g_strdup ((*argv)[i]);
+      gtk_argv[gtk_argc] = NULL;
     }
 
   if (gtk_debug_flags & GTK_DEBUG_UPDATES)
@@ -720,8 +838,10 @@ gtk_init_check (int	 *argc,
 
 #ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, GTK_LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE "-properties", GTK_LOCALEDIR);
 #    ifdef HAVE_BIND_TEXTDOMAIN_CODESET
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  bind_textdomain_codeset (GETTEXT_PACKAGE "-properties", "UTF-8");
 #    endif
 #endif  
 
@@ -739,13 +859,6 @@ gtk_init_check (int	 *argc,
     }
   }
 
-  /* Initialize the default visual and colormap to be
-   *  used in creating widgets. (We want to use the system
-   *  defaults so as to be nice to the colormap).
-   */
-  gtk_visual = gdk_visual_get_system ();
-  gtk_colormap = gdk_colormap_get_system ();
-
   gtk_type_init (0);
   _gtk_accel_map_init ();  
   _gtk_rc_init ();
@@ -754,36 +867,91 @@ gtk_init_check (int	 *argc,
    */
   gtk_initialized = TRUE;
 
-  /* initialize gtk modules
+  display_manager = gdk_display_manager_get ();
+  g_signal_connect (display_manager, "notify::default-display",
+		    G_CALLBACK (default_display_notify_cb), NULL);
+  g_signal_connect (display_manager, "display-opened",
+		    G_CALLBACK (display_opened_cb), NULL);
+
+  /* initialize multhead aware gtk modules; for other modules,
+   * we wait until we have a display open;
    */
   for (slist = gtk_modules; slist; slist = slist->next)
     {
       if (slist->data)
 	{
-	  GtkModuleInitFunc modinit;
-	  
-	  modinit = slist->data;
-	  modinit (argc, argv);
+	  GtkModuleInfo *info = slist->data;
+
+	  if (info->display_init_func)
+	    info->init_func (argc, argv);
 	}
     }
-  g_slist_free (gtk_modules);
   
   return TRUE;
 }
 
+#undef gtk_init_check
+
+/**
+ * gtk_init_check:
+ * @argc: Address of the <parameter>argc</parameter> parameter of your 
+ *   main() function. Changed if any arguments were handled.
+ * @argv: Address of the <parameter>argv</parameter> parameter of main(). 
+ *   Any parameters understood by gtk_init() are stripped before return.
+ * 
+ * This function does the same work as gtk_init() with only 
+ * a single change: It does not terminate the program if the GUI can't be 
+ * initialized. Instead it returns %FALSE on failure.
+ *
+ * This way the application can fall back to some other means of communication 
+ * with the user - for example a curses or command line interface.
+ * 
+ * Return value: %TRUE if the GUI has been successfully initialized, 
+ *               %FALSE otherwise.
+ **/
+gboolean
+gtk_init_check (int	 *argc,
+		char   ***argv)
+{
+  if (!gtk_parse_args (argc, argv))
+    return FALSE;
+
+  return gdk_display_open_default_libgtk_only () != NULL;
+}
+
 #undef gtk_init
 
+/**
+ * gtk_init:
+ * @argc: Address of the <parameter>argc</parameter> parameter of your 
+ *   main() function. Changed if any arguments were handled.
+ * @argv: Address of the <parameter>argv</parameter> parameter of main(). 
+ *   Any parameters understood by gtk_init() are stripped before return.
+ * 
+ * Call this function before using any other GTK+ functions in your GUI
+ * applications.  It will initialize everything needed to operate the 
+ * toolkit and parses some standard command line options. @argc and 
+ * @argv are adjusted accordingly so your own code will 
+ * never see those standard arguments.
+ *
+ * <note><para>
+ * This function will terminate your program if it was unable to initialize 
+ * the GUI for some reason. If you want your program to fall back to a 
+ * textual interface you want to call gtk_init_check() instead.
+ * </para></note>
+ **/
 void
 gtk_init (int *argc, char ***argv)
 {
   if (!gtk_init_check (argc, argv))
     {
-      g_warning ("cannot open display: %s", gdk_get_display ());
+      const char *display_name_arg = gdk_get_display_arg_name ();
+      g_warning ("cannot open display: %s", display_name_arg ? display_name_arg : " ");
       exit (1);
     }
 }
 
-#ifdef G_OS_WIN32
+#ifdef G_PLATFORM_WIN32
 
 static void
 check_sizeof_GtkWindow (size_t sizeof_GtkWindow)
@@ -793,23 +961,48 @@ check_sizeof_GtkWindow (size_t sizeof_GtkWindow)
 	     "The code using GTK+ thinks GtkWindow is of different\n"
              "size than it actually is in this build of GTK+.\n"
 	     "On Windows, this probably means that you have compiled\n"
-	     "your code with gcc without the -fnative-struct switch.");
+	     "your code with gcc without the -fnative-struct\n"
+	     "(or -mms-bitfields) switch, or that you are using\n"
+	     "an unsupported compiler.");
+}
+
+/* In GTK+ 2.0 the GtkWindow struct actually is the same size in
+ * gcc-compiled code on Win32 whether compiled with -fnative-struct or
+ * not. Unfortunately this wan't noticed until after GTK+ 2.0.1. So,
+ * from GTK+ 2.0.2 on, check some other struct, too, where the use of
+ * -fnative-struct still matters. GtkBox is one such.
+ */
+static void
+check_sizeof_GtkBox (size_t sizeof_GtkBox)
+{
+  if (sizeof_GtkBox != sizeof (GtkBox))
+    g_error ("Incompatible build!\n"
+	     "The code using GTK+ thinks GtkBox is of different\n"
+             "size than it actually is in this build of GTK+.\n"
+	     "On Windows, this probably means that you have compiled\n"
+	     "your code with gcc without the -fnative-struct\n"
+	     "(or -mms-bitfields) switch, or that you are using\n"
+	     "an unsupported compiler.");
 }
 
 /* These two functions might get more checks added later, thus pass
  * in the number of extra args.
  */
 void
-gtk_init_abi_check (int *argc, char ***argv, int num_checks, size_t sizeof_GtkWindow)
+gtk_init_abi_check (int *argc, char ***argv, int num_checks, size_t sizeof_GtkWindow, size_t sizeof_GtkBox)
 {
   check_sizeof_GtkWindow (sizeof_GtkWindow);
+  if (num_checks >= 2)
+    check_sizeof_GtkBox (sizeof_GtkBox);
   gtk_init (argc, argv);
 }
 
 gboolean
-gtk_init_check_abi_check (int *argc, char ***argv, int num_checks, size_t sizeof_GtkWindow)
+gtk_init_check_abi_check (int *argc, char ***argv, int num_checks, size_t sizeof_GtkWindow, size_t sizeof_GtkBox)
 {
   check_sizeof_GtkWindow (sizeof_GtkWindow);
+  if (num_checks >= 2)
+    check_sizeof_GtkBox (sizeof_GtkBox);
   return gtk_init_check (argc, argv);
 }
 
@@ -818,9 +1011,7 @@ gtk_init_check_abi_check (int *argc, char ***argv, int num_checks, size_t sizeof
 void
 gtk_exit (gint errorcode)
 {
-  /* Only if "gtk" has been initialized should we de-initialize.
-   */
-  gdk_exit (errorcode);
+  exit (errorcode);
 }
 
 
@@ -842,25 +1033,91 @@ gtk_exit (gint errorcode)
  * <literal>setlocale (LC_ALL, "")</literal> but also takes care of the 
  * locale specific setup of the windowing system used by GDK.
  * 
- * Return value: a string corresponding to the locale set, as with the
- * C library function <function>setlocale()</function>.
+ * Returns: a string corresponding to the locale set, typically in the
+ * form lang_COUNTRY, where lang is an ISO-639 language code, and
+ * COUNTRY is an ISO-3166 country code. On Unix, this form matches the
+ * result of the setlocale(); it is also used on other machines, such as 
+ * Windows, where the C library returns a different result. The string is 
+ * owned by GTK+ and should not be modified or freed.
  **/
-gchar*
+gchar *
 gtk_set_locale (void)
 {
   return gdk_set_locale ();
 }
 
+/**
+ * _gtk_get_lc_ctype:
+ *
+ * Return the Unix-style locale string for the language currently in
+ * effect. On Unix systems, this is the return value from
+ * <literal>setlocale(LC_CTYPE, NULL)</literal>, and the user can
+ * affect this through the environment variables LC_ALL, LC_CTYPE or
+ * LANG (checked in that order). The locale strings typically is in
+ * the form lang_COUNTRY, where lang is an ISO-639 language code, and
+ * COUNTRY is an ISO-3166 country code. For instance, sv_FI for
+ * Swedish as written in Finland or pt_BR for Portuguese as written in
+ * Brazil.
+ * 
+ * On Windows, the C library doesn't use any such environment
+ * variables, and setting them won't affect the behaviour of functions
+ * like ctime(). The user sets the locale through the Regional Options 
+ * in the Control Panel. The C library (in the setlocale() function) 
+ * does not use country and language codes, but country and language 
+ * names spelled out in English. 
+ * However, this function does check the above environment
+ * variables, and does return a Unix-style locale string based on
+ * either said environment variables or the thread's current locale.
+ *
+ * Return value: a dynamically allocated string, free with g_free().
+ */
+
+gchar *
+_gtk_get_lc_ctype (void)
+{
+#ifdef G_OS_WIN32
+  /* Somebody might try to set the locale for this process using the
+   * LANG or LC_ environment variables. The Microsoft C library
+   * doesn't know anything about them. You set the locale in the
+   * Control Panel. Setting these env vars won't have any affect on
+   * locale-dependent C library functions like ctime(). But just for
+   * kicks, do obey LC_ALL, LC_CTYPE and LANG in GTK. (This also makes
+   * it easier to test GTK and Pango in various default languages, you
+   * don't have to clickety-click in the Control Panel, you can simply
+   * start the program with LC_ALL=something on the command line.)
+   */
+  gchar *p;
+
+  p = getenv ("LC_ALL");
+  if (p != NULL)
+    return g_strdup (p);
+
+  p = getenv ("LC_CTYPE");
+  if (p != NULL)
+    return g_strdup (p);
+
+  p = getenv ("LANG");
+  if (p != NULL)
+    return g_strdup (p);
+
+  return g_win32_getlocale ();
+#else
+  return g_strdup (setlocale (LC_CTYPE, NULL));
+#endif
+}
+
 /**
  * gtk_get_default_language:
  *
- * Returns the ISO language code for the default language currently in
+ * Returns the #PangoLanguage for the default language currently in
  * effect. (Note that this can change over the life of an
  * application.)  The default language is derived from the current
  * locale. It determines, for example, whether GTK+ uses the
- * right-to-left or left-to-right text direction.
+ * right-to-left or left-to-right text direction. See
+ * _gtk_get_lc_ctype() for notes on behaviour on Windows.
  * 
- * Return value: the default language as an allocated string, must be freed
+ * Return value: the default language as a #PangoLanguage, must not be
+ * freed
  **/
 PangoLanguage *
 gtk_get_default_language (void)
@@ -869,7 +1126,7 @@ gtk_get_default_language (void)
   PangoLanguage *result;
   gchar *p;
   
-  lang = g_strdup (setlocale (LC_CTYPE, NULL));
+  lang = _gtk_get_lc_ctype ();
   p = strchr (lang, '.');
   if (p)
     *p = '\0';
@@ -893,7 +1150,7 @@ gtk_main (void)
 
   gtk_main_loop_level++;
   
-  loop = g_main_new (TRUE);
+  loop = g_main_loop_new (NULL, TRUE);
   main_loops = g_slist_prepend (main_loops, loop);
 
   tmp_list = functions = init_functions;
@@ -909,10 +1166,10 @@ gtk_main (void)
     }
   g_list_free (functions);
 
-  if (g_main_is_running (main_loops->data))
+  if (g_main_loop_is_running (main_loops->data))
     {
       GDK_THREADS_LEAVE ();
-      g_main_run (loop);
+      g_main_loop_run (loop);
       GDK_THREADS_ENTER ();
       gdk_flush ();
     }
@@ -956,7 +1213,7 @@ gtk_main (void)
 	      
   main_loops = g_slist_remove (main_loops, loop);
 
-  g_main_destroy (loop);
+  g_main_loop_unref (loop);
 
   gtk_main_loop_level--;
 }
@@ -972,16 +1229,16 @@ gtk_main_quit (void)
 {
   g_return_if_fail (main_loops != NULL);
 
-  g_main_quit (main_loops->data);
+  g_main_loop_quit (main_loops->data);
 }
 
-gint
+gboolean
 gtk_events_pending (void)
 {
   gboolean result;
   
   GDK_THREADS_LEAVE ();  
-  result = g_main_pending ();
+  result = g_main_context_pending (NULL);
   GDK_THREADS_ENTER ();
 
   return result;
@@ -991,11 +1248,11 @@ gboolean
 gtk_main_iteration (void)
 {
   GDK_THREADS_LEAVE ();
-  g_main_iteration (TRUE);
+  g_main_context_iteration (NULL, TRUE);
   GDK_THREADS_ENTER ();
 
   if (main_loops)
-    return !g_main_is_running (main_loops->data);
+    return !g_main_loop_is_running (main_loops->data);
   else
     return TRUE;
 }
@@ -1004,20 +1261,22 @@ gboolean
 gtk_main_iteration_do (gboolean blocking)
 {
   GDK_THREADS_LEAVE ();
-  g_main_iteration (blocking);
+  g_main_context_iteration (NULL, blocking);
   GDK_THREADS_ENTER ();
 
   if (main_loops)
-    return !g_main_is_running (main_loops->data);
+    return !g_main_loop_is_running (main_loops->data);
   else
     return TRUE;
 }
 
 /* private libgtk to libgdk interfaces
  */
-gboolean gdk_pointer_grab_info_libgtk_only  (GdkWindow **grab_window,
+gboolean gdk_pointer_grab_info_libgtk_only  (GdkDisplay *display,
+					     GdkWindow **grab_window,
 					     gboolean   *owner_events);
-gboolean gdk_keyboard_grab_info_libgtk_only (GdkWindow **grab_window,
+gboolean gdk_keyboard_grab_info_libgtk_only (GdkDisplay *display,
+					     GdkWindow **grab_window,
 					     gboolean   *owner_events);
 
 static void
@@ -1032,8 +1291,8 @@ rewrite_events_translate (GdkWindow *old_window,
   gdk_window_get_origin	(old_window, &old_origin_x, &old_origin_y);
   gdk_window_get_origin	(new_window, &new_origin_x, &new_origin_y);
 
-  *x += new_origin_x - old_origin_x;
-  *y += new_origin_y - old_origin_y;
+  *x += old_origin_x - new_origin_x;
+  *y += old_origin_y - new_origin_y;
 }
 
 static GdkEvent *
@@ -1091,6 +1350,7 @@ rewrite_event_for_grabs (GdkEvent *event)
   GdkWindow *grab_window;
   GtkWidget *event_widget, *grab_widget;
   gboolean owner_events;
+  GdkDisplay *display;
 
   switch (event->type)
     {
@@ -1102,14 +1362,16 @@ rewrite_event_for_grabs (GdkEvent *event)
     case GDK_MOTION_NOTIFY:
     case GDK_PROXIMITY_IN:
     case GDK_PROXIMITY_OUT:
-      if (!gdk_pointer_grab_info_libgtk_only (&grab_window, &owner_events) ||
+      display = gdk_drawable_get_display (event->proximity.window);
+      if (!gdk_pointer_grab_info_libgtk_only (display, &grab_window, &owner_events) ||
 	  !owner_events)
 	return NULL;
       break;
 
     case GDK_KEY_PRESS:
     case GDK_KEY_RELEASE:
-      if (!gdk_keyboard_grab_info_libgtk_only (&grab_window, &owner_events) ||
+      display = gdk_drawable_get_display (event->key.window);
+      if (!gdk_keyboard_grab_info_libgtk_only (display, &grab_window, &owner_events) ||
 	  !owner_events)
 	return NULL;
       break;
@@ -1170,6 +1432,18 @@ gtk_main_do_event (GdkEvent *event)
   if (next_event)
     gdk_event_free (next_event);
 
+  if (event->type == GDK_SETTING)
+    {
+      _gtk_settings_handle_event (&event->setting);
+      return;
+    }
+
+  if (event->type == GDK_OWNER_CHANGE)
+    {
+      _gtk_clipboard_handle_event (&event->owner_change);
+      return;
+    }
+
   /* Find the widget which got the event. We store the widget
    *  in the user_data field of GdkWindow's.
    *  Ignore the event if we don't have a widget for it, except
@@ -1187,10 +1461,8 @@ gtk_main_do_event (GdkEvent *event)
 	   * them specially
 	   */
       if (event->type == GDK_PROPERTY_NOTIFY)
-	gtk_selection_incr_event (event->any.window,
-				  &event->property);
-      else if (event->type == GDK_SETTING)
-	_gtk_settings_handle_event (&event->setting);
+	_gtk_selection_incr_event (event->any.window,
+				   &event->property);
 
       return;
     }
@@ -1247,11 +1519,11 @@ gtk_main_do_event (GdkEvent *event)
       break;
       
     case GDK_DELETE:
-      gtk_widget_ref (event_widget);
+      g_object_ref (event_widget);
       if ((!window_group->grabs || gtk_widget_get_toplevel (window_group->grabs->data) == event_widget) &&
 	  !gtk_widget_event (event_widget, event))
 	gtk_widget_destroy (event_widget);
-      gtk_widget_unref (event_widget);
+      g_object_unref (event_widget);
       break;
       
     case GDK_DESTROY:
@@ -1260,11 +1532,11 @@ gtk_main_do_event (GdkEvent *event)
        */
       if (!event_widget->parent)
 	{
-	  gtk_widget_ref (event_widget);
+	  g_object_ref (event_widget);
 	  if (!gtk_widget_event (event_widget, event) &&
 	      GTK_WIDGET_REALIZED (event_widget))
 	    gtk_widget_destroy (event_widget);
-	  gtk_widget_unref (event_widget);
+	  g_object_unref (event_widget);
 	}
       break;
       
@@ -1415,14 +1687,14 @@ gtk_grab_notify_foreach (GtkWidget *child,
 
   if (was_grabbed != is_grabbed)
     {
-      g_object_ref (G_OBJECT (child));
-      
-      gtk_signal_emit_by_name (GTK_OBJECT (child), "grab_notify", was_grabbed);
+      g_object_ref (child);
+
+      _gtk_widget_grab_notify (child, was_grabbed);
       
       if (GTK_IS_CONTAINER (child))
 	gtk_container_foreach (GTK_CONTAINER (child), gtk_grab_notify_foreach, info);
       
-      g_object_unref (G_OBJECT (child));
+      g_object_unref (child);
     }
 }
 
@@ -1481,7 +1753,7 @@ gtk_grab_add (GtkWidget *widget)
 
       was_grabbed = (group->grabs != NULL);
       
-      gtk_widget_ref (widget);
+      g_object_ref (widget);
       group->grabs = g_slist_prepend (group->grabs, widget);
 
       gtk_grab_notify (group, widget, FALSE);
@@ -1514,7 +1786,7 @@ gtk_grab_remove (GtkWidget *widget)
       group = gtk_main_get_window_group (widget);
       group->grabs = g_slist_remove (group->grabs, widget);
       
-      gtk_widget_unref (widget);
+      g_object_unref (widget);
 
       gtk_grab_notify (group, widget, TRUE);
     }
@@ -1552,7 +1824,7 @@ gtk_key_snooper_install (GtkKeySnoopFunc snooper,
 }
 
 void
-gtk_key_snooper_remove (guint		 snooper_id)
+gtk_key_snooper_remove (guint snooper_id)
 {
   GtkKeySnooperData *data = NULL;
   GSList *slist;
@@ -1568,7 +1840,10 @@ gtk_key_snooper_remove (guint		 snooper_id)
       data = NULL;
     }
   if (data)
-    key_snoopers = g_slist_remove (key_snoopers, data);
+    {
+      key_snoopers = g_slist_remove (key_snoopers, data);
+      g_free (data);
+    }
 }
 
 static gint
@@ -1650,10 +1925,10 @@ gtk_quit_add_destroy (guint              main_level,
 
   object_p = g_new (GtkObject*, 1);
   *object_p = object;
-  gtk_signal_connect (object,
-		      "destroy",
-		      GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-		      object_p);
+  g_signal_connect (object,
+		    "destroy",
+		    G_CALLBACK (gtk_widget_destroyed),
+		    object_p);
   gtk_quit_add (main_level, (GtkFunction) gtk_quit_destructor, object_p);
 }
 
@@ -1781,7 +2056,7 @@ guint
 gtk_idle_add (GtkFunction function,
 	      gpointer	  data)
 {
-  return g_idle_add_full (GTK_PRIORITY_DEFAULT, function, data, NULL);
+  return g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, function, data, NULL);
 }
 
 guint	    
@@ -1856,7 +2131,7 @@ gtk_invoke_idle_timeout (gpointer data)
   GtkArg args[1];
   gint ret_val = FALSE;
   args[0].name = NULL;
-  args[0].type = GTK_TYPE_BOOL;
+  args[0].type = G_TYPE_BOOLEAN;
   args[0].d.pointer_data = &ret_val;
   closure->marshal (NULL, closure->data,  0, args);
   return ret_val;
@@ -1870,13 +2145,13 @@ gtk_invoke_input (gpointer	    data,
   GtkClosure *closure = data;
 
   GtkArg args[3];
-  args[0].type = GTK_TYPE_INT;
+  args[0].type = G_TYPE_INT;
   args[0].name = NULL;
   GTK_VALUE_INT (args[0]) = source;
   args[1].type = GDK_TYPE_INPUT_CONDITION;
   args[1].name = NULL;
   GTK_VALUE_FLAGS (args[1]) = condition;
-  args[2].type = GTK_TYPE_NONE;
+  args[2].type = G_TYPE_NONE;
   args[2].name = NULL;
 
   closure->marshal (NULL, closure->data, 2, args);
@@ -1976,7 +2251,7 @@ gtk_quit_invoke_function (GtkQuitFunction *quitf)
       gint ret_val = FALSE;
 
       args[0].name = NULL;
-      args[0].type = GTK_TYPE_BOOL;
+      args[0].type = G_TYPE_BOOLEAN;
       args[0].d.pointer_data = &ret_val;
       ((GtkCallbackMarshal) quitf->marshal) (NULL,
 					     quitf->data,
@@ -2019,7 +2294,7 @@ gtk_propagate_event (GtkWidget *widget,
   
   handled_event = FALSE;
 
-  gtk_widget_ref (widget);
+  g_object_ref (widget);
       
   if ((event->type == GDK_KEY_PRESS) ||
       (event->type == GDK_KEY_RELEASE))
@@ -2063,21 +2338,30 @@ gtk_propagate_event (GtkWidget *widget,
       while (TRUE)
 	{
 	  GtkWidget *tmp;
-	  
-	  handled_event = !GTK_WIDGET_IS_SENSITIVE (widget) || gtk_widget_event (widget, event);
+
+	  /* Scroll events are special cased here because it
+	   * feels wrong when scrolling a GtkViewport, say,
+	   * to have children of the viewport eat the scroll
+	   * event
+	   */
+	  if (!GTK_WIDGET_IS_SENSITIVE (widget))
+	    handled_event = event->type != GDK_SCROLL;
+	  else
+	    handled_event = gtk_widget_event (widget, event);
+	      
 	  tmp = widget->parent;
-	  gtk_widget_unref (widget);
+	  g_object_unref (widget);
 
 	  widget = tmp;
 	  
 	  if (!handled_event && widget)
-	    gtk_widget_ref (widget);
+	    g_object_ref (widget);
 	  else
 	    break;
 	}
     }
   else
-    gtk_widget_unref (widget);
+    g_object_unref (widget);
 }
 
 #if 0
