@@ -17,12 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* This file implements most of the work of the ICCM selection protocol.
+/* This file implements most of the work of the ICCCM selection protocol.
  * The code was written after an intensive study of the equivalent part
  * of John Ousterhout's Tk toolkit, and does many things in much the 
  * same way.
  *
- * The one thing in the ICCM that isn't fully supported here (or in Tk)
+ * The one thing in the ICCCM that isn't fully supported here (or in Tk)
  * is side effects targets. For these to be handled properly, MULTIPLE
  * targets need to be done in the order specified. This cannot be
  * guaranteed with the way we do things, since if we are doing INCR
@@ -51,6 +51,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <stdarg.h>
 #include <string.h>
 #include "gdk.h"
@@ -58,15 +59,23 @@
 #include "gtkmain.h"
 #include "gtkselection.h"
 
-/* #define DEBUG_SELECTION */
+#ifdef GDK_WINDOWING_X11
+#include "x11/gdkx.h"
+#endif
+
+#undef DEBUG_SELECTION
 
 /* Maximum size of a sent chunk, in bytes. Also the default size of
    our buffers */
-#ifdef GDK_WINDOWING_WIN32
-/* No chunks on Win32 */
-#define GTK_SELECTION_MAX_SIZE G_MAXINT
+#ifdef GDK_WINDOWING_X11
+#define GTK_SELECTION_MAX_SIZE(display)                                 \
+  MIN(262144,                                                           \
+      XExtendedMaxRequestSize (GDK_DISPLAY_XDISPLAY (display)) == 0     \
+       ? XMaxRequestSize (GDK_DISPLAY_XDISPLAY (display)) - 100         \
+       : XExtendedMaxRequestSize (GDK_DISPLAY_XDISPLAY (display)) - 100)
 #else
-#define GTK_SELECTION_MAX_SIZE 4000
+/* No chunks on Win32 */
+#define GTK_SELECTION_MAX_SIZE(display) G_MAXINT
 #endif
 
 #define IDLE_ABORT_TIME 300
@@ -156,7 +165,7 @@ static GList *current_incrs = NULL;
 static GList *current_selections = NULL;
 
 static GdkAtom gtk_selection_atoms[LAST_ATOM];
-static const char *gtk_selection_handler_key = "gtk-selection-handlers";
+static const char gtk_selection_handler_key[] = "gtk-selection-handlers";
 
 /****************
  * Target Lists *
@@ -213,9 +222,9 @@ gtk_target_list_unref (GtkTargetList *list)
 
 void 
 gtk_target_list_add (GtkTargetList *list,
-		     GdkAtom            target,
-		     guint              flags,
-		     guint              info)
+		     GdkAtom        target,
+		     guint          flags,
+		     guint          info)
 {
   GtkTargetPair *pair;
 
@@ -229,6 +238,61 @@ gtk_target_list_add (GtkTargetList *list,
   list->list = g_list_append (list->list, pair);
 }
 
+static GdkAtom utf8_atom;
+static GdkAtom text_atom;
+static GdkAtom ctext_atom;
+static GdkAtom text_plain_atom;
+static GdkAtom text_plain_utf8_atom;
+static GdkAtom text_plain_locale_atom;
+
+static void 
+init_atoms (void)
+{
+  gchar *tmp;
+  const gchar *charset;
+
+  if (!utf8_atom)
+    {
+      utf8_atom = gdk_atom_intern ("UTF8_STRING", FALSE);
+      text_atom = gdk_atom_intern ("TEXT", FALSE);
+      ctext_atom = gdk_atom_intern ("COMPOUND_TEXT", FALSE);
+      text_plain_atom = gdk_atom_intern ("text/plain", FALSE);
+      text_plain_utf8_atom = gdk_atom_intern ("text/plain;charset=utf-8", FALSE);
+      g_get_charset (&charset);
+      tmp = g_strdup_printf ("text/plain;charset=%s", charset);
+      text_plain_locale_atom = gdk_atom_intern (tmp, FALSE);
+      g_free (tmp);
+    }
+}
+
+/**
+ * gtk_target_list_add_text_targets:
+ * @list: a #GtkTargetList
+ * 
+ * Adds the text targets supported by #GtkSelection to
+ * the target list. The targets are added with both flags
+ * and info being zero.
+ * 
+ * Since: 2.6
+ **/
+void 
+gtk_target_list_add_text_targets (GtkTargetList *list)
+{
+  g_return_if_fail (list != NULL);
+  
+  init_atoms ();
+
+  /* Keep in sync with gtk_selection_data_targets_include_text()
+   */
+  gtk_target_list_add (list, utf8_atom, 0, 0);  
+  gtk_target_list_add (list, ctext_atom, 0, 0);  
+  gtk_target_list_add (list, text_atom, 0, 0);  
+  gtk_target_list_add (list, GDK_TARGET_STRING, 0, 0);  
+  gtk_target_list_add (list, text_plain_utf8_atom, 0, 0);  
+  gtk_target_list_add (list, text_plain_locale_atom, 0, 0);  
+  gtk_target_list_add (list, text_plain_atom, 0, 0);  
+}
+
 void               
 gtk_target_list_add_table (GtkTargetList        *list,
 			   const GtkTargetEntry *targets,
@@ -300,12 +364,14 @@ gtk_target_list_find (GtkTargetList *list,
  * @display: the #Gdkdisplay where the selection is set 
  * @widget: new selection owner (a #GdkWidget), or %NULL.
  * @selection: an interned atom representing the selection to claim.
- * @time: timestamp with which to claim the selection
+ * @time_: timestamp with which to claim the selection
  *
  * Claim ownership of a given selection for a particular widget, or,
  * if @widget is %NULL, release ownership of the selection.
  *
  * Return value: TRUE if the operation succeeded 
+ * 
+ * Since: 2.2
  */
 gboolean
 gtk_selection_owner_set_for_display (GdkDisplay   *display,
@@ -319,6 +385,7 @@ gtk_selection_owner_set_for_display (GdkDisplay   *display,
   GdkWindow *window;
 
   g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
+  g_return_val_if_fail (selection != GDK_NONE, FALSE);
   g_return_val_if_fail (widget == NULL || GTK_WIDGET_REALIZED (widget), FALSE);
   g_return_val_if_fail (widget == NULL || gtk_widget_get_display (widget) == display, FALSE);
   
@@ -399,7 +466,7 @@ gtk_selection_owner_set_for_display (GdkDisplay   *display,
  * gtk_selection_owner_set:
  * @widget:  a #GtkWidget, or %NULL.
  * @selection:  an interned atom representing the selection to claim
- * @time: timestamp with which to claim the selection
+ * @time_: timestamp with which to claim the selection
  * 
  * Claims ownership of a given selection for a particular widget,
  * or, if @widget is %NULL, release ownership of the selection.
@@ -414,6 +481,7 @@ gtk_selection_owner_set (GtkWidget *widget,
   GdkDisplay *display;
   
   g_return_val_if_fail (widget == NULL || GTK_WIDGET_REALIZED (widget), FALSE);
+  g_return_val_if_fail (selection != GDK_NONE, FALSE);
 
   if (widget)
     display = gtk_widget_get_display (widget);
@@ -518,6 +586,9 @@ gtk_selection_clear_targets (GtkWidget *widget,
   GList *tmp_list;
   GList *lists;
 
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (selection != GDK_NONE);
+
   lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
   tmp_list = lists;
@@ -547,7 +618,8 @@ gtk_selection_add_target (GtkWidget	    *widget,
 {
   GtkTargetList *list;
 
-  g_return_if_fail (widget != NULL);
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (selection != GDK_NONE);
 
   list = gtk_selection_target_list_get (widget, selection);
   gtk_target_list_add (list, target, 0, info);
@@ -560,8 +632,9 @@ gtk_selection_add_targets (GtkWidget            *widget,
 			   guint                 ntargets)
 {
   GtkTargetList *list;
-  
-  g_return_if_fail (widget != NULL);
+
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (selection != GDK_NONE);
   g_return_if_fail (targets != NULL);
   
   list = gtk_selection_target_list_get (widget, selection);
@@ -659,7 +732,8 @@ gtk_selection_convert (GtkWidget *widget,
   GdkWindow *owner_window;
   GdkDisplay *display;
   
-  g_return_val_if_fail (widget != NULL, FALSE);
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  g_return_val_if_fail (selection != GDK_NONE, FALSE);
   
   if (initialize)
     gtk_selection_init ();
@@ -734,7 +808,7 @@ gtk_selection_convert (GtkWidget *widget,
   
   current_retrievals = g_list_append (current_retrievals, info);
   gdk_selection_convert (widget->window, selection, target, time);
-  gtk_timeout_add (1000, (GtkFunction) gtk_selection_retrieval_timeout, info);
+  g_timeout_add (1000, (GSourceFunc) gtk_selection_retrieval_timeout, info);
   
   return TRUE;
 }
@@ -784,19 +858,196 @@ gtk_selection_data_set (GtkSelectionData *selection_data,
   selection_data->length = length;
 }
 
-static GdkAtom utf8_atom;
-static GdkAtom text_atom;
-static GdkAtom ctext_atom;
-
-static void 
-init_atoms (void)
+static gboolean
+selection_set_string (GtkSelectionData *selection_data,
+		      const gchar      *str,
+		      gint              len)
 {
-  if (!utf8_atom)
+  gchar *tmp = g_strndup (str, len);
+  gchar *latin1 = gdk_utf8_to_string_target (tmp);
+  g_free (tmp);
+  
+  if (latin1)
     {
-      utf8_atom = gdk_atom_intern ("UTF8_STRING", FALSE);
-      text_atom = gdk_atom_intern ("TEXT", FALSE);
-      ctext_atom = gdk_atom_intern ("COMPOUND_TEXT", FALSE);
+      gtk_selection_data_set (selection_data,
+			      GDK_SELECTION_TYPE_STRING,
+			      8, latin1, strlen (latin1));
+      g_free (latin1);
+      
+      return TRUE;
     }
+  else
+    return FALSE;
+}
+
+static gboolean
+selection_set_compound_text (GtkSelectionData *selection_data,
+			     const gchar      *str,
+			     gint              len)
+{
+  gchar *tmp;
+  guchar *text;
+  GdkAtom encoding;
+  gint format;
+  gint new_length;
+  gboolean result = FALSE;
+  
+  tmp = g_strndup (str, len);
+  if (gdk_utf8_to_compound_text_for_display (selection_data->display, tmp,
+					     &encoding, &format, &text, &new_length))
+    {
+      gtk_selection_data_set (selection_data, encoding, format, text, new_length);
+      gdk_free_compound_text (text);
+      
+      result = TRUE;
+    }
+
+  g_free (tmp);
+
+  return result;
+}
+
+/* Normalize \r and \n into \r\n
+ */
+static gchar *
+normalize_to_crlf (const gchar *str, 
+		   gint         len)
+{
+  GString *result = g_string_sized_new (len);
+  const gchar *p = str;
+
+  while (1)
+    {
+      if (*p == '\n')
+	g_string_append_c (result, '\r');
+
+      if (*p == '\r')
+	{
+	  g_string_append_c (result, *p);
+	  p++;
+	  if (*p != '\n')
+	    g_string_append_c (result, '\n');
+	}
+
+      if (*p == '\0')
+	break;
+
+      g_string_append_c (result, *p);
+      p++;
+    }
+
+  return g_string_free (result, FALSE);  
+}
+
+/* Normalize \r and \r\n into \n
+ */
+static gchar *
+normalize_to_lf (gchar *str, 
+		 gint   len)
+{
+  GString *result = g_string_sized_new (len);
+  const gchar *p = str;
+
+  while (1)
+    {
+      if (*p == '\r')
+	{
+	  p++;
+	  if (*p != '\n')
+	    g_string_append_c (result, '\n');
+	}
+
+      if (*p == '\0')
+	break;
+
+      g_string_append_c (result, *p);
+      p++;
+    }
+
+  return g_string_free (result, FALSE);  
+}
+
+static gboolean
+selection_set_text_plain (GtkSelectionData *selection_data,
+			  const gchar      *str,
+			  gint              len)
+{
+  const gchar *charset = NULL;
+  gchar *result;
+  GError *error = NULL;
+
+  result = normalize_to_crlf (str, len);
+  if (selection_data->target == text_plain_atom)
+    charset = "ASCII";
+  else if (selection_data->target == text_plain_locale_atom)
+    g_get_charset (&charset);
+
+  if (charset)
+    {
+      gchar *tmp = result;
+      result = g_convert_with_fallback (tmp, -1, 
+					charset, "UTF-8", 
+					NULL, NULL, NULL, &error);
+      g_free (tmp);
+    }
+
+  if (!result)
+    {
+      g_warning ("Error converting from UTF-8 to %s: %s",
+		 charset, error->message);
+      g_error_free (error);
+      
+      return FALSE;
+    }
+  
+  gtk_selection_data_set (selection_data,
+			  selection_data->target, 
+			  8, result, strlen (result));
+  g_free (result);
+  
+  return TRUE;
+}
+
+static gchar *
+selection_get_text_plain (GtkSelectionData *selection_data)
+{
+  const gchar *charset = NULL;
+  gchar *str, *result;
+  gint len;
+  GError *error = NULL;
+
+  str = g_strdup (selection_data->data);
+  len = selection_data->length;
+  
+  if (selection_data->type == text_plain_atom)
+    charset = "ISO-8859-1";
+  else if (selection_data->type == text_plain_locale_atom)
+    g_get_charset (&charset);
+
+  if (charset)
+    {
+      gchar *tmp = str;
+      str = g_convert_with_fallback (tmp, len, 
+				     charset, "UTF-8", 
+				     NULL, NULL, &len, &error);
+      g_free (tmp);
+    }
+
+  if (!str)
+    {
+      g_warning ("Error converting from %s to UTF-8: %s",
+		 charset, error->message);
+      g_error_free (error);
+      
+      return NULL;
+    }
+
+
+  result = normalize_to_lf (str, len);
+
+  g_free (str);
+
+  return result;
 }
 
 /**
@@ -817,8 +1068,6 @@ gtk_selection_data_set_text (GtkSelectionData     *selection_data,
 			     const gchar          *str,
 			     gint                  len)
 {
-  gboolean result = FALSE;
-  
   if (len < 0)
     len = strlen (str);
   
@@ -829,48 +1078,28 @@ gtk_selection_data_set_text (GtkSelectionData     *selection_data,
       gtk_selection_data_set (selection_data,
 			      utf8_atom,
 			      8, (guchar *)str, len);
-      result = TRUE;
+      return TRUE;
     }
   else if (selection_data->target == GDK_TARGET_STRING)
     {
-      gchar *tmp = g_strndup (str, len);
-      gchar *latin1 = gdk_utf8_to_string_target (tmp);
-      g_free (tmp);
-
-      if (latin1)
-	{
-	  gtk_selection_data_set (selection_data,
-				  GDK_SELECTION_TYPE_STRING,
-				  8, latin1, strlen (latin1));
-	  g_free (latin1);
-	  
-	  result = TRUE;
-	}
-
+      return selection_set_string (selection_data, str, len);
     }
   else if (selection_data->target == ctext_atom ||
 	   selection_data->target == text_atom)
     {
-      gchar *tmp;
-      guchar *text;
-      GdkAtom encoding;
-      gint format;
-      gint new_length;
-
-      tmp = g_strndup (str, len);
-      if (gdk_utf8_to_compound_text_for_display (selection_data->display, tmp,
-						 &encoding, &format, &text, &new_length))
-	{
-	  gtk_selection_data_set (selection_data, encoding, format, text, new_length);
-	  gdk_free_compound_text (text);
-
-	  result = TRUE;
-	}
-
-      g_free (tmp);
+      if (selection_set_compound_text (selection_data, str, len))
+	return TRUE;
+      else if (selection_data->target == text_atom)
+	return selection_set_string (selection_data, str, len);
     }
-  
-  return result;
+  else if (selection_data->target == text_plain_atom ||
+	   selection_data->target == text_plain_utf8_atom ||
+	   selection_data->target == text_plain_locale_atom)
+    {
+      return selection_set_text_plain (selection_data, str, len);
+    }
+
+  return FALSE;
 }
 
 /**
@@ -911,6 +1140,13 @@ gtk_selection_data_get_text (GtkSelectionData *selection_data)
 	g_free (list[i]);
       g_free (list);
     }
+  else if (selection_data->length >= 0 &&
+	   (selection_data->type == text_plain_atom ||
+	    selection_data->type == text_plain_utf8_atom ||
+	    selection_data->type == text_plain_locale_atom))
+    {
+      result = selection_get_text_plain (selection_data);
+    }
 
   return result;
 }
@@ -976,14 +1212,19 @@ gtk_selection_data_targets_include_text (GtkSelectionData *selection_data)
   gint i;
   gboolean result = FALSE;
 
+  init_atoms ();
+
   if (gtk_selection_data_get_targets (selection_data, &targets, &n_targets))
     {
       for (i=0; i < n_targets; i++)
 	{
-	  if (targets[i] == gdk_atom_intern ("STRING", FALSE) ||
-	      targets[i] == gdk_atom_intern ("TEXT", FALSE) ||
-	      targets[i] == gdk_atom_intern ("COMPOUND_TEXT", FALSE) ||
-	      targets[i] == gdk_atom_intern ("UTF8_STRING", FALSE))
+	  if (targets[i] == utf8_atom ||
+	      targets[i] == text_atom ||
+	      targets[i] == GDK_TARGET_STRING ||
+	      targets[i] == ctext_atom ||
+	      targets[i] == text_plain_atom ||
+	      targets[i] == text_plain_utf8_atom ||
+	      targets[i] == text_plain_locale_atom)
 	    result = TRUE;
 	}
 
@@ -1018,12 +1259,15 @@ gtk_selection_init (void)
  * @event: the event
  * 
  * The default handler for the GtkWidget::selection_clear_event
- * signal. Instead of calling this function, chain up from
- * your selection_clear_event handler. Calling this function
- * from any other context is illegal. This function will
- * be deprecated in future versions of GTK+.
+ * signal. 
  * 
  * Return value: %TRUE if the event was handled, otherwise false
+ * 
+ * Since: 2.2
+ *
+ * Deprecated: Instead of calling this function, chain up from
+ * your selection_clear_event handler. Calling this function
+ * from any other context is illegal. 
  **/
 gboolean
 gtk_selection_clear (GtkWidget         *widget,
@@ -1072,14 +1316,17 @@ gboolean
 _gtk_selection_request (GtkWidget *widget,
 			GdkEventSelection *event)
 {
+  GdkDisplay *display = gtk_widget_get_display (widget);
   GtkIncrInfo *info;
   GList *tmp_list;
-  guchar *mult_atoms;
   int i;
-  
+  gulong selection_max_size;
+
   if (initialize)
     gtk_selection_init ();
   
+  selection_max_size = GTK_SELECTION_MAX_SIZE (display);
+
   /* Check if we own selection */
   
   tmp_list = current_selections;
@@ -1106,10 +1353,10 @@ _gtk_selection_request (GtkWidget *widget,
   
   /* Create GdkWindow structure for the requestor */
   
-  info->requestor = gdk_window_lookup_for_display (gtk_widget_get_display (widget),
+  info->requestor = gdk_window_lookup_for_display (display,
 						   event->requestor);
   if (!info->requestor)
-    info->requestor = gdk_window_foreign_new_for_display (gtk_widget_get_display (widget),
+    info->requestor = gdk_window_foreign_new_for_display (display,
 							  event->requestor);
   
   /* Determine conversions we need to perform */
@@ -1117,17 +1364,18 @@ _gtk_selection_request (GtkWidget *widget,
   if (event->target == gtk_selection_atoms[MULTIPLE])
     {
       GdkAtom  type;
+      guchar  *mult_atoms;
       gint     format;
       gint     length;
       
       mult_atoms = NULL;
       
       gdk_error_trap_push ();
-      if (!gdk_property_get (info->requestor, event->property, 0, /* AnyPropertyType */
-			     0, GTK_SELECTION_MAX_SIZE, FALSE,
+      if (!gdk_property_get (info->requestor, event->property, GDK_NONE, /* AnyPropertyType */
+			     0, selection_max_size, FALSE,
 			     &type, &format, &length, &mult_atoms))
 	{
-	  gdk_selection_send_notify_for_display (gtk_widget_get_display (widget),
+	  gdk_selection_send_notify_for_display (display,
 						 event->requestor, 
 						 event->selection,
 						 event->target, 
@@ -1138,14 +1386,37 @@ _gtk_selection_request (GtkWidget *widget,
 	  return TRUE;
 	}
       gdk_error_trap_pop ();
-      
-      info->num_conversions = length / (2*sizeof (GdkAtom));
-      info->conversions = g_new (GtkIncrConversion, info->num_conversions);
-      
-      for (i=0; i<info->num_conversions; i++)
+
+      /* This is annoying; the ICCCM doesn't specify the property type
+       * used for the property contents, so the autoconversion for
+       * ATOM / ATOM_PAIR in GDK doesn't work properly.
+       */
+#ifdef GDK_WINDOWING_X11
+      if (type != GDK_SELECTION_TYPE_ATOM &&
+	  type != gdk_atom_intern ("ATOM_PAIR", FALSE))
 	{
-	  info->conversions[i].target = ((GdkAtom *)mult_atoms)[2*i];
-	  info->conversions[i].property = ((GdkAtom *)mult_atoms)[2*i+1];
+	  info->num_conversions = length / (2*sizeof (glong));
+	  info->conversions = g_new (GtkIncrConversion, info->num_conversions);
+	  
+	  for (i=0; i<info->num_conversions; i++)
+	    {
+	      info->conversions[i].target = gdk_x11_xatom_to_atom_for_display (display,
+									       ((glong *)mult_atoms)[2*i]);
+	      info->conversions[i].property = gdk_x11_xatom_to_atom_for_display (display,
+										 ((glong *)mult_atoms)[2*i + 1]);
+	    }
+	}
+      else
+#endif
+	{
+	  info->num_conversions = length / (2*sizeof (GdkAtom));
+	  info->conversions = g_new (GtkIncrConversion, info->num_conversions);
+	  
+	  for (i=0; i<info->num_conversions; i++)
+	    {
+	      info->conversions[i].target = ((GdkAtom *)mult_atoms)[2*i];
+	      info->conversions[i].property = ((GdkAtom *)mult_atoms)[2*i+1];
+	    }
 	}
     }
   else				/* only a single conversion */
@@ -1154,7 +1425,6 @@ _gtk_selection_request (GtkWidget *widget,
       info->num_conversions = 1;
       info->conversions[0].target = event->target;
       info->conversions[0].property = event->property;
-      mult_atoms = (guchar *)info->conversions;
     }
   
   /* Loop through conversions and determine which of these are big
@@ -1172,16 +1442,16 @@ _gtk_selection_request (GtkWidget *widget,
       
 #ifdef DEBUG_SELECTION
       g_message ("Selection %ld, target %ld (%s) requested by 0x%x (property = %ld)",
-		 event->selection, info->conversions[i].target,
+		 event->selection, 
+		 info->conversions[i].target,
 		 gdk_atom_name (info->conversions[i].target),
-		 event->requestor, event->property);
+		 event->requestor, info->conversions[i].property);
 #endif
       
       gtk_selection_invoke_handler (widget, &data, event->time);
       
       if (data.length < 0)
 	{
-	  ((GdkAtom *)mult_atoms)[2*i+1] = GDK_NONE;
 	  info->conversions[i].property = GDK_NONE;
 	  continue;
 	}
@@ -1190,9 +1460,13 @@ _gtk_selection_request (GtkWidget *widget,
       
       items = data.length / gtk_selection_bytes_per_item (data.format);
       
-      if (data.length > GTK_SELECTION_MAX_SIZE)
+      if (data.length > selection_max_size)
 	{
 	  /* Sending via INCR */
+#ifdef DEBUG_SELECTION
+	  g_message ("Target larger (%d) than max. request size (%ld), sending incrementally\n",
+		     data.length, selection_max_size);
+#endif
 	  
 	  info->conversions[i].offset = 0;
 	  info->conversions[i].data = data;
@@ -1236,17 +1510,24 @@ _gtk_selection_request (GtkWidget *widget,
 			     gdk_window_get_events (info->requestor) |
 			     GDK_PROPERTY_CHANGE_MASK);
       current_incrs = g_list_append (current_incrs, info);
-      gtk_timeout_add (1000, (GtkFunction)gtk_selection_incr_timeout, info);
+      g_timeout_add (1000, (GSourceFunc) gtk_selection_incr_timeout, info);
     }
   
   /* If it was a MULTIPLE request, set the property to indicate which
      conversions succeeded */
   if (event->target == gtk_selection_atoms[MULTIPLE])
     {
+      GdkAtom *mult_atoms = g_new (GdkAtom, 2 * info->num_conversions);
+      for (i = 0; i < info->num_conversions; i++)
+	{
+	  mult_atoms[2*i] = info->conversions[i].target;
+	  mult_atoms[2*i+1] = info->conversions[i].property;
+	}
+      
       gdk_property_change (info->requestor, event->property,
 			   gdk_atom_intern ("ATOM_PAIR", FALSE), 32, 
 			   GDK_PROP_MODE_REPLACE,
-			   mult_atoms, 2*info->num_conversions);
+			   (guchar *)mult_atoms, 2*info->num_conversions);
       g_free (mult_atoms);
     }
 
@@ -1305,6 +1586,7 @@ _gtk_selection_incr_event (GdkWindow	   *window,
   GtkIncrInfo *info = NULL;
   gint num_bytes;
   guchar *buffer;
+  gulong selection_max_size;
   
   int i;
   
@@ -1314,7 +1596,9 @@ _gtk_selection_incr_event (GdkWindow	   *window,
 #ifdef DEBUG_SELECTION
   g_message ("PropertyDelete, property %ld", event->atom);
 #endif
-  
+
+  selection_max_size = GTK_SELECTION_MAX_SIZE (gdk_drawable_get_display (window));  
+
   /* Now find the appropriate ongoing INCR */
   tmp_list = current_incrs;
   while (tmp_list)
@@ -1352,10 +1636,10 @@ _gtk_selection_incr_event (GdkWindow	   *window,
 	      buffer = info->conversions[i].data.data + 
 		info->conversions[i].offset;
 	      
-	      if (num_bytes > GTK_SELECTION_MAX_SIZE)
+	      if (num_bytes > selection_max_size)
 		{
-		  num_bytes = GTK_SELECTION_MAX_SIZE;
-		  info->conversions[i].offset += GTK_SELECTION_MAX_SIZE;
+		  num_bytes = selection_max_size;
+		  info->conversions[i].offset += selection_max_size;
 		}
 	      else
 		info->conversions[i].offset = -2;
@@ -1386,7 +1670,6 @@ _gtk_selection_incr_event (GdkWindow	   *window,
 	      info->conversions[i].offset = -1;
 	    }
 	}
-      break;
     }
   
   /* Check if we're finished with all the targets */
@@ -1838,9 +2121,12 @@ gtk_selection_default_handler (GtkWidget	*widget,
       data->type = GDK_SELECTION_TYPE_ATOM;
       data->format = 32;
       data->length = count * sizeof (GdkAtom);
-      
-      p = g_new (GdkAtom, count);
+
+      /* selection data is always terminated by a trailing \0
+       */
+      p = g_malloc (data->length + 1);
       data->data = (guchar *)p;
+      data->data[data->length] = '\0';
       
       *p++ = gtk_selection_atoms[TIMESTAMP];
       *p++ = gtk_selection_atoms[TARGETS];
