@@ -210,6 +210,7 @@ struct _GtkTreeViewPrivate
   guint disable_popdown : 1;
   
   guint hover_selection : 1;
+  guint imcontext_changed : 1;
 
   gint selected_iter;
   gint search_column;
@@ -219,8 +220,14 @@ struct _GtkTreeViewPrivate
   GtkDestroyNotify search_destroy;
   GtkWidget *search_window;
   GtkWidget *search_entry;
+  guint search_entry_changed_id;
+  guint typeselect_flush_timeout;
 
   gint prev_width;
+
+  GtkTreeViewRowSeparatorFunc row_separator_func;
+  gpointer row_separator_data;
+  GtkDestroyNotify row_separator_destroy;
 };
 
 #ifdef __GNUC__
@@ -319,8 +326,6 @@ void         _gtk_tree_view_queue_draw_node           (GtkTreeView       *tree_v
 						       GtkRBTree         *tree,
 						       GtkRBNode         *node,
 						       GdkRectangle      *clip_rect);
-void         _gtk_tree_view_set_hover_selection       (GtkTreeView       *tree_view,
-						       gboolean           hover);
 
 void _gtk_tree_view_column_realize_button   (GtkTreeViewColumn *column);
 void _gtk_tree_view_column_unrealize_button (GtkTreeViewColumn *column);
@@ -382,7 +387,6 @@ void              _gtk_tree_view_column_get_neighbor_sizes (GtkTreeViewColumn *c
 							    gint              *right);
 
 
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
